<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\TestAttribute;
use App\TestResult;
use App\IdpResult;
use App\Libraries\EarcUtils;
use Carbon\Carbon; 
use Illuminate\Support\Str;

class ServiceProviderController extends Controller
{
    /**
     * Handles the SP part
     *
     */
    public function __invoke(Request $request)
    {
        if ($request->server->has('SHIB_Shib-Identity-Provider')){
                $idp_entityid_param = "&entityID=" . $request->server->get('SHIB_Shib-Identity-Provider');
        }else{
                $idp_entityid_param="";
        }
        $attribute = array();
        foreach ($request->server as $key => $value ){
            if ((str_starts_with($key, "SHIB_") && !(str_starts_with($key,"SHIB_Shib")))){
                $values = explode(';', $value);
                foreach ($values as $v){
                    $attribute[str_replace("SHIB_", "", $key)][] = $v;
                }
            }
        }
        $current_sp = str_replace(".", "_", $request->server->get('SHIB_Shib-Application-ID'));
        $request->session()->put("released_attributes." . $current_sp , $attribute);
        if (count($request->session()->get('remaining_sps')) > 0){
            $remaining_sps = $request->session()->get('remaining_sps');
            $redirect_url = "https://" . parse_url($remaining_sps[0], PHP_URL_HOST) . 
                            "/Shibboleth.sso/Login?target=" . 
                            urlencode("https://" . parse_url($remaining_sps[0], PHP_URL_HOST)."/sp") . 
                            $idp_entityid_param;
            $request->session()->put('remaining_sps',array_slice($remaining_sps,1));
            return redirect($redirect_url);
        }else{
            $test_id = $this->store_results($request);
            $request->session()->forget('remaining_sps');
            $request->session()->forget('test_allowed');
            $request->session()->forget('released_attributes');
            return redirect(config('app.url')."/results/".$test_id);
        }
    }

    private function store_results($request){
        //Store the attributes
        $idp_entity_id = $request->server->get('SHIB_Shib-Identity-Provider');
        $now = Carbon::now();
        $idp_result = new IdpResult;
        $idp_metadata = EARCUtils::getIdpMetadata($request->server->get('SHIB_Shib-Identity-Provider'));
        $idp_result->idp_entity_id = $idp_entity_id;
        $idp_result->idp_name = $idp_metadata['name']['en'];
        $idp_result->registration_authority = EARCUtils::getRegistrationAuthority($idp_metadata); 
        $test_id = Str::random(48);
        $idp_result->test_id = $test_id;
        $idp_result->save();
        $released_attributes = $request->session()->get('released_attributes');
        foreach ($released_attributes as $current_sp => $attributes){
            if (!$current_sp) {
                continue;
            }
            $current_sp = "https://".str_replace('_',".", $current_sp) . "/shibboleth";
            $attributes = EarcUtils::handleEptid($attributes);
            foreach ($attributes as $attr_name => $attr_values){
                $test_attr = new TestAttribute;
                $test_attr->attr_name = $attr_name;
                $test_attr->attr_value_multiplicity = count($attr_values);
                $test_attr->sp_entityid = $current_sp;
                $test_attr->test_id = $test_id;
                $test_attr->save();
            }
            $test_result = new TestResult;
            $test_result->test_id = $test_id;
            $test_result->sp_entityid = $current_sp;
            $verdict = EarcUtils::calculateVerdictForAnSP($current_sp, $attributes, $idp_entity_id);
            $test_result->test_mark = $verdict['mark'];
            $test_result->test_comment = $verdict['text'];
            $test_result->save();
        }
        return $test_id;
    }

}
